@@ -7,16 +7,25 @@ Google API.
 
 """
 
+from http import HTTPStatus
+from typing import Type
+
 from google.auth.transport.requests import AuthorizedSession
 
-from .exceptions import APIError, SpreadsheetNotFound
+from .exceptions import APIError, SpreadsheetNotFound, UnSupportedExportFormat
 from .spreadsheet import Spreadsheet
 from .urls import (
-    DRIVE_FILES_API_V2_URL,
+    DRIVE_FILES_API_V3_COMMENTS_URL,
     DRIVE_FILES_API_V3_URL,
     DRIVE_FILES_UPLOAD_API_V2_URL,
 )
-from .utils import convert_credentials, extract_id_from_url, finditem
+from .utils import (
+    ExportFormat,
+    MimeType,
+    convert_credentials,
+    extract_id_from_url,
+    finditem,
+)
 
 
 class Client:
@@ -39,6 +48,8 @@ class Client:
         else:
             self.session = session
 
+        self.timeout = None
+
     def login(self):
         from google.auth.transport.requests import Request
 
@@ -46,6 +57,15 @@ class Client:
 
         self.session.headers.update({"Authorization": "Bearer %s" % self.auth.token})
 
+    def set_timeout(self, timeout):
+        """How long to wait for the server to send
+        data before giving up, as a float, or a ``(connect timeout,
+        read timeout)`` tuple.
+
+        Value for ``timeout`` is in seconds (s).
+        """
+        self.timeout = timeout
+
     def request(
         self,
         method,
@@ -63,6 +83,7 @@ class Client:
             data=data,
             files=files,
             headers=headers,
+            timeout=self.timeout,
         )
 
         if response.ok:
@@ -71,11 +92,21 @@ class Client:
             raise APIError(response)
 
     def list_spreadsheet_files(self, title=None, folder_id=None):
+        """List all the spreadsheet files
+
+        Will list all spreadsheet files owned by/shared with this user account.
+
+        :param str title: Filter only spreadsheet files with this title
+        :param str folder_id: Only look for spreadsheet files in this folder
+            The parameter ``folder_id`` can be obtained from the URL when looking at
+            a folder in a web browser as follow:
+            ``https://drive.google.com/drive/u/0/folders/<folder_id>``
+        """
         files = []
         page_token = ""
         url = DRIVE_FILES_API_V3_URL
 
-        q = 'mimeType="application/vnd.google-apps.spreadsheet"'
+        q = 'mimeType="{}"'.format(MimeType.google_sheets)
         if title:
             q += ' and name = "{}"'.format(title)
         if folder_id:
@@ -184,7 +215,7 @@ class Client:
         """
         payload = {
             "name": title,
-            "mimeType": "application/vnd.google-apps.spreadsheet",
+            "mimeType": MimeType.google_sheets,
         }
 
         params = {
@@ -198,7 +229,46 @@ class Client:
         spreadsheet_id = r.json()["id"]
         return self.open_by_key(spreadsheet_id)
 
-    def copy(self, file_id, title=None, copy_permissions=False, folder_id=None):
+    def export(self, file_id, format=ExportFormat.PDF):
+        """Export the spreadsheet in the given format.
+
+        :param str file_id: The key of the spreadsheet to export
+
+        :param str format: The format of the resulting file.
+            Possible values are:
+
+                * ``ExportFormat.PDF``
+                * ``ExportFormat.EXCEL``
+                * ``ExportFormat.CSV``
+                * ``ExportFormat.OPEN_OFFICE_SHEET``
+                * ``ExportFormat.TSV``
+                * ``ExportFormat.ZIPPED_HTML``
+
+            See `ExportFormat`_ in the Drive API.
+
+        :returns bytes: The content of the exported file.
+
+        .. _ExportFormat: https://developers.google.com/drive/api/guides/ref-export-formats
+        """
+
+        if format not in ExportFormat:
+            raise UnSupportedExportFormat
+
+        url = "{}/{}/export".format(DRIVE_FILES_API_V3_URL, file_id)
+
+        params = {"mimeType": format}
+
+        r = self.request("get", url, params=params)
+        return r.content
+
+    def copy(
+        self,
+        file_id,
+        title=None,
+        copy_permissions=False,
+        folder_id=None,
+        copy_comments=True,
+    ):
         """Copies a spreadsheet.
 
         :param str file_id: A key of a spreadsheet to copy.
@@ -210,6 +280,9 @@ class Client:
         :param str folder_id: Id of the folder where we want to save
             the spreadsheet.
 
+        :param bool copy_comments: (optional) If True, copy the comments from
+            the original spreadsheet to the new spreadsheet.
+
         :returns: a :class:`~gspread.models.Spreadsheet` instance.
 
         .. versionadded:: 3.1.0
@@ -231,11 +304,11 @@ class Client:
            when you try to copy a spreadsheet.
 
         """
-        url = "{}/{}/copy".format(DRIVE_FILES_API_V2_URL, file_id)
+        url = "{}/{}/copy".format(DRIVE_FILES_API_V3_URL, file_id)
 
         payload = {
-            "title": title,
-            "mimeType": "application/vnd.google-apps.spreadsheet",
+            "name": title,
+            "mimeType": MimeType.google_sheets,
         }
 
         if folder_id is not None:
@@ -264,6 +337,30 @@ class Client:
                 except Exception:
                     pass
 
+        if copy_comments is True:
+            source_url = DRIVE_FILES_API_V3_COMMENTS_URL % (file_id)
+            page_token = ""
+            comments = []
+            params = {
+                "fields": "comments/content,comments/anchor,nextPageToken",
+                "includeDeleted": False,
+                "pageSize": 100,  # API limit to maximum 100
+            }
+
+            while page_token is not None:
+                params["pageToken"] = page_token
+                res = self.request("get", source_url, params=params).json()
+
+                comments.extend(res["comments"])
+                page_token = res.get("nextPageToken", None)
+
+            destination_url = DRIVE_FILES_API_V3_COMMENTS_URL % (new_spreadsheet.id)
+            # requesting some fields in the response is mandatory from the API.
+            # choose 'id' randomly out of all the fields, but no need to use it for now.
+            params = {"fields": "id"}
+            for comment in comments:
+                self.request("post", destination_url, json=comment, params=params)
+
         return new_spreadsheet
 
     def del_spreadsheet(self, file_id):
@@ -296,6 +393,10 @@ class Client:
            replaces the contents of the first worksheet.
 
         """
+        # Make sure we send utf-8
+        if type(data) is str:
+            data = data.encode("utf-8")
+
         headers = {"Content-Type": "text/csv"}
         url = "{}/{}".format(DRIVE_FILES_UPLOAD_API_V2_URL, file_id)
 
@@ -316,12 +417,27 @@ class Client:
 
         :param str file_id: a spreadsheet ID (aka file ID).
         """
-        url = "{}/{}/permissions".format(DRIVE_FILES_API_V2_URL, file_id)
+        url = "{}/{}/permissions".format(DRIVE_FILES_API_V3_URL, file_id)
 
-        params = {"supportsAllDrives": True}
-        r = self.request("get", url, params=params)
+        params = {
+            "supportsAllDrives": True,
+            "fields": "nextPageToken,permissions",
+        }
 
-        return r.json()["items"]
+        token = ""
+
+        permissions = []
+
+        while token is not None:
+            if token:
+                params["pageToken"] = token
+
+            r = self.request("get", url, params=params).json()
+            permissions.extend(r["permissions"])
+
+            token = r.get("nextPageToken", None)
+
+        return permissions
 
     def insert_permission(
         self,
@@ -343,13 +459,15 @@ class Client:
             Allowed values are: ``user``, ``group``, ``domain``, ``anyone``
         :param str role: (optional) The primary role for this user.
             Allowed values are: ``owner``, ``writer``, ``reader``
-        :param str notify: (optional) Whether to send an email to the target
+        :param bool notify: (optional) Whether to send an email to the target
             user/domain.
         :param str email_message: (optional) An email message to be sent
             if ``notify=True``.
         :param bool with_link: (optional) Whether the link is required for this
             permission to be active.
 
+        :returns dict: the newly created permission
+
         Examples::
 
             # Give write permissions to otto@example.com
@@ -372,22 +490,28 @@ class Client:
 
         """
 
-        url = "{}/{}/permissions".format(DRIVE_FILES_API_V2_URL, file_id)
-
+        url = "{}/{}/permissions".format(DRIVE_FILES_API_V3_URL, file_id)
         payload = {
-            "value": value,
             "type": perm_type,
             "role": role,
             "withLink": with_link,
         }
-
         params = {
-            "sendNotificationEmails": notify,
-            "emailMessage": email_message,
             "supportsAllDrives": "true",
         }
 
-        self.request("post", url, json=payload, params=params)
+        if perm_type == "domain":
+            payload["domain"] = value
+        elif perm_type in {"user", "group"}:
+            payload["emailAddress"] = value
+            params["sendNotificationEmail"] = notify
+            params["emailMessage"] = email_message
+        elif perm_type == "anyone":
+            pass
+        else:
+            raise ValueError("Invalid permission type: {}".format(perm_type))
+
+        return self.request("post", url, json=payload, params=params)
 
     def remove_permission(self, file_id, permission_id):
         """Deletes a permission from a file.
@@ -396,8 +520,79 @@ class Client:
         :param str permission_id: an ID for the permission.
         """
         url = "{}/{}/permissions/{}".format(
-            DRIVE_FILES_API_V2_URL, file_id, permission_id
+            DRIVE_FILES_API_V3_URL, file_id, permission_id
         )
 
         params = {"supportsAllDrives": True}
         self.request("delete", url, params=params)
+
+
+class BackoffClient(Client):
+    """BackoffClient is a gspread client with exponential
+    backoff retries.
+
+    In case a request fails due to some API rate limits,
+    it will wait for some time, then retry the request.
+
+    This can help by trying the request after some time and
+    prevent the application from failing (by raising an APIError exception).
+
+    .. Warning::
+        This Client is not production ready yet.
+        Use it at your own risk !
+
+    .. note::
+        To use with the `auth` module, make sure to pass this backoff
+        client factory using the ``client_factory`` parameter of the
+        method used.
+
+    .. note::
+        Currently known issues are:
+
+        * will retry exponentially even when the error should
+          raise instantly. Due to the Drive API that raises
+          403 (Forbidden) errors for forbidden access and
+          for api rate limit exceeded."""
+
+    _HTTP_ERROR_CODES = [
+        HTTPStatus.FORBIDDEN,  # Drive API return a 403 Forbidden on usage rate limit exceeded
+        HTTPStatus.REQUEST_TIMEOUT,  # in case of a timeout
+        HTTPStatus.TOO_MANY_REQUESTS,  # sheet API usage rate limit exceeded
+    ]
+    _NR_BACKOFF = 0
+    _MAX_BACKOFF = 128  # arbitrary maximum backoff
+    _MAX_BACKOFF_REACHED = False  # Stop after reaching _MAX_BACKOFF
+
+    def request(self, *args, **kwargs):
+        try:
+            return super().request(*args, **kwargs)
+        except APIError as err:
+            data = err.response.json()
+            code = data["error"]["code"]
+
+            # check if error should retyr
+            if code in self._HTTP_ERROR_CODES and self._MAX_BACKOFF_REACHED is False:
+                self._NR_BACKOFF += 1
+                wait = min(2**self._NR_BACKOFF, self._MAX_BACKOFF)
+
+                if wait >= self._MAX_BACKOFF:
+                    self._MAX_BACKOFF_REACHED = True
+
+                import time
+
+                time.sleep(wait)
+
+                # make the request again
+                response = self.request(*args, **kwargs)
+
+                # reset counters for next time
+                self._NR_BACKOFF = 0
+                self._MAX_BACKOFF_REACHED = False
+
+                return response
+
+            # failed too many times, raise APIEerror
+            raise err
+
+
+ClientFactory = Type[Client]
