@@ -8,6 +8,7 @@ This module contains common spreadsheets' models.
 
 from .exceptions import WorksheetNotFound
 from .urls import (
+    DRIVE_FILES_API_V3_URL,
     SPREADSHEET_BATCH_UPDATE_URL,
     SPREADSHEET_DRIVE_URL,
     SPREADSHEET_SHEETS_COPY_TO_URL,
@@ -19,7 +20,7 @@ from .urls import (
     SPREADSHEET_VALUES_CLEAR_URL,
     SPREADSHEET_VALUES_URL,
 )
-from .utils import finditem, quote
+from .utils import ExportFormat, finditem, quote
 from .worksheet import Worksheet
 
 
@@ -65,7 +66,7 @@ class Spreadsheet:
 
     @property
     def lastUpdateTime(self):
-        """Spreadsheet Creation time."""
+        """Spreadsheet last updated time."""
         try:
             return self._properties["modifiedTime"]
         except KeyError:
@@ -93,6 +94,16 @@ class Spreadsheet:
             stacklevel=2,
         )
 
+    @property
+    def timezone(self):
+        """Spreadsheet timeZone"""
+        return self._properties["timeZone"]
+
+    @property
+    def locale(self):
+        """Spreadsheet locale"""
+        return self._properties["locale"]
+
     @property
     def sheet1(self):
         """Shortcut property for getting the first worksheet."""
@@ -109,7 +120,7 @@ class Spreadsheet:
         )
 
     def batch_update(self, body):
-        """Lower-level method that directly calls `spreadsheets.batchUpdate <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/batchUpdate>`_.
+        """Lower-level method that directly calls `spreadsheets/<ID>:batchUpdate <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/batchUpdate>`_.
 
         :param dict body: `Request body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/batchUpdate#request-body>`_.
         :returns: `Response body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets/batchUpdate#response-body>`_.
@@ -124,13 +135,13 @@ class Spreadsheet:
         return r.json()
 
     def values_append(self, range, params, body):
-        """Lower-level method that directly calls `spreadsheets.values.append <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append>`_.
+        """Lower-level method that directly calls `spreadsheets/<ID>/values:append <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append>`_.
 
         :param str range: The `A1 notation <https://developers.google.com/sheets/api/guides/concepts#a1_notation>`_
                           of a range to search for a logical table of data. Values will be appended after the last row of the table.
         :param dict params: `Query parameters <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#query-parameters>`_.
-        :param dict body: `Request body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#request-body>`_.
-        :returns: `Response body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/append#response-body>`_.
+        :param dict body: `Request body`_.
+        :returns: `Response body`_.
         :rtype: dict
 
         .. versionadded:: 3.0
@@ -140,10 +151,10 @@ class Spreadsheet:
         return r.json()
 
     def values_clear(self, range):
-        """Lower-level method that directly calls `spreadsheets.values.clear <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/clear>`_.
+        """Lower-level method that directly calls `spreadsheets/<ID>/values:clear <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/clear>`_.
 
         :param str range: The `A1 notation <https://developers.google.com/sheets/api/guides/concepts#a1_notation>`_ of the values to clear.
-        :returns: `Response body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/clear#response-body>`_.
+        :returns: `Response body`_.
         :rtype: dict
 
         .. versionadded:: 3.0
@@ -158,11 +169,11 @@ class Spreadsheet:
         return r.json()
 
     def values_get(self, range, params=None):
-        """Lower-level method that directly calls `spreadsheets.values.get <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get>`_.
+        """Lower-level method that directly calls `spreadsheets/<ID>/values/<range> <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get>`_.
 
         :param str range: The `A1 notation <https://developers.google.com/sheets/api/guides/concepts#a1_notation>`_ of the values to retrieve.
-        :param dict params: (optional) `Query parameters <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get#query-parameters>`_.
-        :returns: `Response body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get#response-body>`_.
+        :param dict params: (optional) `Query parameters`_.
+        :returns: `Response body`_.
         :rtype: dict
 
         .. versionadded:: 3.0
@@ -172,11 +183,11 @@ class Spreadsheet:
         return r.json()
 
     def values_batch_get(self, ranges, params=None):
-        """Lower-level method that directly calls `spreadsheets.values.batchGet <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/batchGet>`_.
+        """Lower-level method that directly calls `spreadsheets/<ID>/values:batchGet <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/batchGet>`_.
 
         :param ranges: List of ranges in the `A1 notation <https://developers.google.com/sheets/api/guides/concepts#a1_notation>`_ of the values to retrieve.
-        :param dict params: (optional) `Query parameters <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get#query-parameters>`_.
-        :returns: `Response body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/get#response-body>`_.
+        :param dict params: (optional) `Query parameters`_.
+        :returns: `Response body`_.
         :rtype: dict
         """
         if params is None:
@@ -189,12 +200,12 @@ class Spreadsheet:
         return r.json()
 
     def values_update(self, range, params=None, body=None):
-        """Lower-level method that directly calls `spreadsheets.values.update <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/update>`_.
+        """Lower-level method that directly calls `spreadsheets/<ID>/values/<range>`_.
 
         :param str range: The `A1 notation <https://developers.google.com/sheets/api/guides/concepts#a1_notation>`_ of the values to update.
-        :param dict params: (optional) `Query parameters <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/update#query-parameters>`_.
-        :param dict body: (optional) `Request body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/update#request-body>`_.
-        :returns: `Response body <https://developers.google.com/sheets/api/reference/rest/v4/spreadsheets.values/update#response-body>`_.
+        :param dict params: (optional) `Query parameters`_.
+        :param dict body: (optional) `Request body`_.
+        :returns: `Response body`_.
         :rtype: dict
 
         Example::
@@ -250,10 +261,10 @@ class Spreadsheet:
         :param index: An index of a worksheet. Indexes start from zero.
         :type index: int
 
-        :returns: an instance of :class:`gspread.models.Worksheet`.
+        :returns: an instance of :class:`gspread.worksheet.Worksheet`.
 
         :raises:
-            WorksheetNotFound: if can't find the worksheet
+            :class:`~gspread.exceptions.WorksheetNotFound`: if can't find the worksheet
 
         Example. To get third worksheet of a spreadsheet:
 
@@ -274,9 +285,9 @@ class Spreadsheet:
         :param id: The id of a worksheet. it can be seen in the url as the value of the parameter 'gid'.
         :type id: int
 
-        :returns: an instance of :class:`gspread.models.Worksheet`.
+        :returns: an instance of :class:`gspread.worksheet.Worksheet`.
         :raises:
-            WorksheetNotFound: if can't find the worksheet
+            :class:`~gspread.exceptions.WorksheetNotFound`: if can't find the worksheet
 
         Example. To get the worksheet 123456 of a spreadsheet:
 
@@ -295,7 +306,7 @@ class Spreadsheet:
             raise WorksheetNotFound("id {} not found".format(id))
 
     def worksheets(self):
-        """Returns a list of all :class:`worksheets <gspread.models.Worksheet>`
+        """Returns a list of all :class:`worksheets <gspread.worksheet.Worksheet>`
         in a spreadsheet.
         """
         sheet_data = self.fetch_sheet_metadata()
@@ -309,7 +320,7 @@ class Spreadsheet:
                       be returned.
         :type title: str
 
-        :returns: an instance of :class:`gspread.models.Worksheet`.
+        :returns: an instance of :class:`gspread.worksheet.Worksheet`.
 
         :raises:
             WorksheetNotFound: if can't find the worksheet
@@ -341,7 +352,7 @@ class Spreadsheet:
         :param index: Position of the sheet.
         :type index: int
 
-        :returns: a newly created :class:`worksheets <gspread.models.Worksheet>`.
+        :returns: a newly created :class:`worksheets <gspread.worksheet.Worksheet>`.
         """
         body = {
             "requests": [
@@ -392,7 +403,7 @@ class Spreadsheet:
         :param str new_sheet_name: (optional) The name of the new sheet.
                                    If empty, a new name is chosen for you.
 
-        :returns: a newly created :class:`<gspread.models.Worksheet>`.
+        :returns: a newly created :class:`gspread.worksheet.Worksheet`
 
         .. versionadded:: 3.1
         """
@@ -421,21 +432,21 @@ class Spreadsheet:
         """Deletes a worksheet from a spreadsheet.
 
         :param worksheet: The worksheet to be deleted.
-        :type worksheet: :class:`~gspread.Worksheet`
+        :type worksheet: :class:`~gspread.worksheet.Worksheet`
         """
         body = {"requests": [{"deleteSheet": {"sheetId": worksheet.id}}]}
 
         return self.batch_update(body)
 
     def reorder_worksheets(self, worksheets_in_desired_order):
-        """Updates the ``index`` property of each Worksheets to reflect
+        """Updates the ``index`` property of each Worksheet to reflect
         its index in the provided sequence of Worksheets.
 
         :param worksheets_in_desired_order: Iterable of Worksheet objects in desired order.
 
         Note: If you omit some of the Spreadsheet's existing Worksheet objects from
         the provided sequence, those Worksheets will be appended to the end of the sequence
-        in the order that they appear in the list returned by ``Spreadsheet.worksheets()``.
+        in the order that they appear in the list returned by :meth:`gspread.spreadsheet.Spreadsheet.worksheets`.
 
         .. versionadded:: 3.4
         """
@@ -464,7 +475,7 @@ class Spreadsheet:
 
     def share(
         self,
-        value,
+        email_address,
         perm_type,
         role,
         notify=True,
@@ -484,10 +495,9 @@ class Spreadsheet:
                Allowed values are: ``owner``, ``writer``, ``reader``.
         :type role: str
         :param notify: (optional) Whether to send an email to the target user/domain.
-        :type notify: str
+        :type notify: bool
         :param email_message: (optional) The email to be sent if notify=True
         :type email_message: str
-
         :param with_link: (optional) Whether the link is required for this permission
         :type with_link: bool
 
@@ -496,12 +506,12 @@ class Spreadsheet:
             # Give Otto a write permission on this spreadsheet
             sh.share('otto@example.com', perm_type='user', role='writer')
 
-            # Transfer ownership to Otto
-            sh.share('otto@example.com', perm_type='user', role='owner')
+            # Give Otto's family a read permission on this spreadsheet
+            sh.share('otto-familly@example.com', perm_type='group', role='reader')
         """
-        self.client.insert_permission(
+        return self.client.insert_permission(
             self.id,
-            value=value,
+            value=email_address,
             perm_type=perm_type,
             role=role,
             notify=notify,
@@ -509,6 +519,30 @@ class Spreadsheet:
             with_link=with_link,
         )
 
+    def export(self, format=ExportFormat.PDF):
+        """Export the spreadsheet in the given format.
+
+        :param str file_id: A key of a spreadsheet to export
+
+        :param str format: The format of the resulting file.
+            Possible values are:
+
+                ``ExportFormat.PDF``,
+                ``ExportFormat.EXCEL``,
+                ``ExportFormat.CSV``,
+                ``ExportFormat.OPEN_OFFICE_SHEET``,
+                ``ExportFormat.TSV``,
+                and ``ExportFormat.ZIPPED_HTML``.
+
+            See `ExportFormat`_ in the Drive API.
+            Default value is ``ExportFormat.PDF``.
+
+        :returns bytes: The content of the exported file.
+
+        .. _ExportFormat: https://developers.google.com/drive/api/guides/ref-export-formats
+        """
+        return self.client.export(self.id, format)
+
     def list_permissions(self):
         """Lists the spreadsheet's permissions."""
         return self.client.list_permissions(self.id)
@@ -545,12 +579,65 @@ class Spreadsheet:
 
         return filtered_id_list
 
+    def transfer_ownership(self, permission_id):
+        """Transfer the ownership of this file to a new user.
+
+        It is necessary to first create the permission with the new owner's email address,
+        get the permission ID then use this method to transfer the ownership.
+
+        .. note::
+
+           You can list all permissions using :meth:`gspread.spreadsheet.Spreadsheet.list_permissions`.
+
+        .. warning::
+
+           You can only transfer ownership to a new user, you cannot transfer ownership to a group
+           or a domain email address.
+        """
+
+        url = "{}/{}/permissions/{}".format(
+            DRIVE_FILES_API_V3_URL, self.id, permission_id
+        )
+
+        payload = {
+            "role": "writer",  # new owner must be writer in order to accept ownership by editing permissions
+            "pendingOwner": True,
+        }
+
+        return self.client.request("patch", url, json=payload)
+
+    def accept_ownership(self, permission_id):
+        """Accept the pending ownership request on that file.
+
+        It is necessary to edit the permission with the pending ownership.
+
+        .. note::
+
+           You can only accept ownership transfer for the user currently being used.
+        """
+
+        url = "{}/{}/permissions/{}".format(
+            DRIVE_FILES_API_V3_URL,
+            self.id,
+            permission_id,
+        )
+
+        payload = {
+            "role": "owner",
+        }
+
+        params = {
+            "transferOwnership": True,
+        }
+
+        return self.client.request("patch", url, json=payload, params=params)
+
     def named_range(self, named_range):
-        """return a list of :class:`Cell` objects from
+        """return a list of :class:`gspread.cell.Cell` objects from
         the specified named range.
 
-        :param name: A string with a named range value to fecth.
-        :type name: str
+        :param named_range: A string with a named range value to fetch.
+        :type named_range: str
         """
 
         # the function `range` does all necessary actions to get a named range.
@@ -562,3 +649,82 @@ class Spreadsheet:
         return self.fetch_sheet_metadata(params={"fields": "namedRanges"}).get(
             "namedRanges", []
         )
+
+    def update_title(self, title):
+        """Renames the spreadsheet.
+
+        :param str title: A new title.
+        """
+        body = {
+            "requests": [
+                {
+                    "updateSpreadsheetProperties": {
+                        "properties": {"title": title},
+                        "fields": "title",
+                    }
+                }
+            ]
+        }
+
+        response = self.batch_update(body)
+        self._properties["title"] = title
+        return response
+
+    def update_timezone(self, timezone):
+        """Updates the current spreadsheet timezone.
+        Can be any timezone in CLDR format such as "America/New_York"
+        or a custom time zone such as GMT-07:00.
+        """
+
+        body = {
+            "requests": [
+                {
+                    "updateSpreadsheetProperties": {
+                        "properties": {"timeZone": timezone},
+                        "fields": "timeZone",
+                    },
+                },
+            ]
+        }
+
+        return self.batch_update(body)
+
+    def update_locale(self, locale):
+        """Update the locale of the spreadsheet.
+        Can be any of the ISO 639-1 language codes, such as: de, fr, en, ...
+        Or an ISO 639-2 if no ISO 639-1 exists.
+        Or a combination of the ISO language code and country code,
+        such as en_US, de_CH, fr_FR, ...
+
+        .. note::
+            Note: when updating this field, not all locales/languages are supported.
+        """
+
+        body = {
+            "requests": [
+                {
+                    "updateSpreadsheetProperties": {
+                        "properties": {"locale": locale},
+                        "fields": "locale",
+                    },
+                },
+            ]
+        }
+
+        return self.batch_update(body)
+
+    def list_protected_ranges(self, sheetid):
+        """Lists the spreadsheet's protected named ranges"""
+        sheets = self.fetch_sheet_metadata(
+            params={"fields": "sheets.properties,sheets.protectedRanges"}
+        )["sheets"]
+
+        try:
+            sheet = finditem(
+                lambda sheet: sheet["properties"]["sheetId"] == sheetid, sheets
+            )
+
+        except StopIteration:
+            raise WorksheetNotFound("worksheet id {} not found".format(sheetid))
+
+        return sheet.get("protectedRanges", [])
